#import "AppDelegate.h"


@implementation AppDelegate

- (void)awakeFromNib;
{
    // Hook up the static progress indicator to our local one
    // that is connected in IB. This will allows us to update
    // the progress indicator in the callback.
    progressIndicator = localProgressIndicator;
    [progressIndicator setUsesThreadedAnimation:YES];
}

- (IBAction)selectSourceFile:(id)sender;
{
    NSOpenPanel *panel = [NSOpenPanel openPanel];
    
    if( [panel runModalForTypes:nil] == NSOKButton )
    {
        [sourceFilePath setStringValue:[panel filename]];
    }
}

- (IBAction)selectDestinationFile:(id)sender;
{
    NSOpenPanel *panel = [NSOpenPanel openPanel];
    
    // Don't allow the user to choose files, only directories
    // for our destination.
    [panel setCanChooseFiles:NO];
    [panel setCanChooseDirectories:YES];
    
    if( [panel runModalForDirectory:nil file:nil] == NSOKButton )
    {
        [destinationFilePath setStringValue:[panel directory]];
    }
}

- (IBAction)startCopy:(id)sender;
{
    // Use the NSFileManager to obtain the size of our source file in bytes.
    NSFileManager *fileManager = [NSFileManager defaultManager];
    NSDictionary *sourceAttributes = [fileManager fileAttributesAtPath:[sourceFilePath stringValue] traverseLink:YES];
    NSNumber *sourceFileSize;
    
    if (sourceFileSize = [sourceAttributes objectForKey:NSFileSize] )
    {
        // Set the max value to our source file size
        [progressIndicator setMaxValue:(double)[sourceFileSize unsignedLongLongValue]];
    }
    else
    {
        // Couldn't get the file size so we need to bail.
        NSLog(@"Unable to obtain size of file being copied.");
        return;
    }
    [progressIndicator setDoubleValue:0.0];
   
    // Get the current run loop and schedule our callback
    CFRunLoopRef runLoop = CFRunLoopGetCurrent();
    FSFileOperationRef fileOp = FSFileOperationCreate(kCFAllocatorDefault);
        
    OSStatus status = FSFileOperationScheduleWithRunLoop(fileOp, runLoop, kCFRunLoopDefaultMode);
    if( status )
    {
        NSLog(@"Failed to schedule operation with run loop: %@", status);
        return;
    }
    
    // Create a filesystem ref structure for the source and destination and
    // populate them with their respective paths from our NSTextFields.
    FSRef source;
    FSRef destination;
    
    FSPathMakeRef( (const UInt8 *)[[sourceFilePath stringValue] fileSystemRepresentation], &source, NULL );

    Boolean isDir = true;
    FSPathMakeRef( (const UInt8 *)[[destinationFilePath stringValue] fileSystemRepresentation], &destination, &isDir );    
    
    // Start the async copy.
    status = FSCopyObjectAsync (fileOp,
                                &source,
                                &destination, // Full path to destination dir
                                NULL, // Use the same filename as source
                                kFSFileOperationDefaultOptions,
                                statusCallback,
                                1.0,
                                NULL);
    
    CFRelease(fileOp);
    
    if( status )
    {
        NSLog(@"Failed to begin asynchronous object copy: %@", status);
    }
}


@end

static void statusCallback (FSFileOperationRef fileOp,
                                  const FSRef *currentItem,
                                  FSFileOperationStage stage,
                                  OSStatus error,
                                  CFDictionaryRef statusDictionary,
                                  void *info )
{
    NSLog(@"Callback got called.");

    // If the status dictionary is valid, we can grab the current values
    // to display status changes, or in our case to update the progress
    // indicator.
    if (statusDictionary)
    {
        CFNumberRef bytesCompleted;
        
        bytesCompleted = (CFNumberRef) CFDictionaryGetValue(statusDictionary, kFSOperationBytesCompleteKey);

        CGFloat floatBytesCompleted;
        CFNumberGetValue (bytesCompleted, kCFNumberMaxType, &floatBytesCompleted);

        NSLog(@"Copied %d bytes so far.", (unsigned long long)floatBytesCompleted);
        
        [progressIndicator setDoubleValue:(double)floatBytesCompleted];
        [progressIndicator displayIfNeeded];
    }
}
